@@ -1,6 +1,4 @@
 import math
-from metadrive.utils.math_utils import clip
-from panda3d.core import WindowProperties
 import queue
 from collections import deque
 from typing import Tuple
@@ -8,6 +6,7 @@ from typing import Tuple
 import numpy as np
 from direct.controls.InputState import InputState
 from panda3d.core import Vec3, Point3
+from panda3d.core import WindowProperties
 
 from metadrive.constants import CollisionGroup
 from metadrive.engine.engine_utils import get_engine
@@ -58,6 +57,8 @@ class MainCamera:
         self.inputs.watchWithModifiers('down', 's')
         self.inputs.watchWithModifiers('left', 'a')
         self.inputs.watchWithModifiers('right', 'd')
+        self.inputs.watchWithModifiers('left_rotate', '[')
+        self.inputs.watchWithModifiers('right_rotate', ']')
 
         self.engine.accept("wheel_up", self._wheel_up_height)
         self.engine.accept("wheel_down", self._wheel_down_height)
@@ -67,14 +68,16 @@ class MainCamera:
         self.top_down_camera_height = self.engine.global_config["top_down_camera_initial_z"]
         self.camera_x = self.engine.global_config["top_down_camera_initial_x"]
         self.camera_y = self.engine.global_config["top_down_camera_initial_y"]
+        self.camera_rotate = 0
         self.engine.interface.stop_track()
         self.engine.task_manager.add(self._top_down_task, self.TOP_DOWN_TASK_NAME, extraArgs=[], appendTask=True)
 
         # TPP rotate
-        props = WindowProperties()
-        # props.setCursorHidden(True)
-        # props.setMouseMode(WindowProperties.MConfined)
-        self.engine.win.requestProperties(props)
+        if not self.engine.global_config["show_mouse"]:
+            props = WindowProperties()
+            props.setCursorHidden(True)
+            props.setMouseMode(WindowProperties.MConfined)
+            self.engine.win.requestProperties(props)
         self.mouse_rotate = 0
         self.last_mouse_pos = self.engine.mouseWatcherNode.getMouseX() if self.has_mouse else 0
         self.static_timer = 0
@@ -87,6 +90,7 @@ class MainCamera:
             # adjust hpr
             p_pos = panda_position(position)
             self.camera_x, self.camera_y = p_pos[0], p_pos[1]
+            self.camera_rotate = 0
             self.engine.task_manager.add(self._top_down_task, self.TOP_DOWN_TASK_NAME, extraArgs=[], appendTask=True)
 
     def reset(self):
@@ -126,7 +130,8 @@ class MainCamera:
         self.chase_camera_height = self._update_height(self.chase_camera_height)
         self.camera_queue.put(vehicle.chassis.get_pos())
         if not self.FOLLOW_LANE:
-            current_forward_dir = vehicle.system.get_forward_vector()
+            forward_dir = vehicle.system.get_forward_vector()
+            current_forward_dir = forward_dir[0], forward_dir[1]
         else:
             current_forward_dir = self._dir_of_lane(vehicle.navigation.current_ref_lanes[0], vehicle.position)
         self.direction_running_mean.append(current_forward_dir)
@@ -244,6 +249,7 @@ class MainCamera:
             if bird_view_on_current_position:
                 current_pos = self.camera.getPos()
                 self.camera_x, self.camera_y = current_pos[0], current_pos[1]
+                self.camera_rotate = 0
             self.engine.task_manager.add(self._top_down_task, self.TOP_DOWN_TASK_NAME, extraArgs=[], appendTask=True)
 
     def _top_down_task(self, task):
@@ -257,8 +263,15 @@ class MainCamera:
             self.camera_x -= 1.0
         if self.inputs.isSet("right"):
             self.camera_x += 1.0
+
         self.camera.setPos(self.camera_x, self.camera_y, self.top_down_camera_height)
         self.camera.lookAt(self.camera_x, self.camera_y, 0)
+
+        if self.inputs.isSet("right_rotate"):
+            self.camera_rotate += 3
+        if self.inputs.isSet("left_rotate"):
+            self.camera_rotate -= 3
+        self.camera.setH(self.camera_rotate)
         return task.cont
 
     def _update_height(self, height):
