@@ -1,37 +1,36 @@
-import sys
 import time
 from collections import defaultdict
 from typing import Union, Dict, AnyStr, Optional, Tuple
-
+from metadrive.manager.record_manager import RecordManager
+from metadrive.manager.replay_manager import ReplayManager
 import gym
 import numpy as np
 from panda3d.core import PNMImage
 
-from metadrive.component.blocks.first_block import FirstPGBlock
 from metadrive.component.vehicle.base_vehicle import BaseVehicle
-from metadrive.constants import RENDER_MODE_NONE, DEFAULT_AGENT
+from metadrive.constants import RENDER_MODE_NONE, DEFAULT_AGENT, REPLAY_DONE
 from metadrive.engine.base_engine import BaseEngine
 from metadrive.engine.engine_utils import initialize_engine, close_engine, \
     engine_initialized, set_global_random_seed
 from metadrive.manager.agent_manager import AgentManager
+from metadrive.obs.image_obs import ImageStateObservation
 from metadrive.obs.observation_base import ObservationBase
+from metadrive.obs.state_obs import LidarStateObservation
 from metadrive.utils import Config, merge_dicts, get_np_random, concat_step_infos
-from metadrive.utils.utils import auto_termination
 
 BASE_DEFAULT_CONFIG = dict(
-    # ===== Generalization =====
-    start_seed=0,
-    environment_num=1,
 
     # ===== agent =====
     random_agent_model=False,
-    IDM_agent=False,
+    agent_policy=None,
 
     # ===== multi-agent =====
     num_agents=1,  # Note that this can be set to >1 in MARL envs, or set to -1 for as many vehicles as possible.
     is_multi_agent=False,
     allow_respawn=False,
     delay_done=0,  # How many steps for the agent to stay static at the death place after done.
+    # Whether only return single agent-like observation and action space
+    return_single_space=False,
 
     # ===== Action =====
     manual_control=False,
@@ -54,6 +53,7 @@ BASE_DEFAULT_CONFIG = dict(
     top_down_camera_initial_x=0,
     top_down_camera_initial_y=0,
     top_down_camera_initial_z=200,  # height
+    show_logo=True,
 
     # ===== Vehicle =====
     vehicle_config=dict(
@@ -71,19 +71,46 @@ BASE_DEFAULT_CONFIG = dict(
         image_source="rgb_camera",  # take effect when only when offscreen_render == True
 
         # ===== vehicle spawn and destination =====
-        spawn_lane_index=(FirstPGBlock.NODE_1, FirstPGBlock.NODE_2, 0),
+        navigation_module=None,  # a class type for self-defined navigation
+        need_navigation=True,
+        spawn_lane_index=None,
         spawn_longitude=5.0,
         spawn_lateral=0.0,
-        destination_node=None,
+        destination=None,
+        spawn_position_heading=None,
 
         # ==== others ====
         overtake_stat=False,  # we usually set to True when evaluation
         action_check=False,
         random_color=False,
+        random_agent_model=False,  # this will be overwritten by env.config["random_agent_model"]
+
+        # ===== vehicle module config =====
+        lidar=dict(
+            num_lasers=240, distance=50, num_others=0, gaussian_noise=0.0, dropout_prob=0.0, add_others_navi=False
+        ),
+        side_detector=dict(num_lasers=0, distance=50, gaussian_noise=0.0, dropout_prob=0.0),
+        lane_line_detector=dict(num_lasers=0, distance=20, gaussian_noise=0.0, dropout_prob=0.0),
+        show_lidar=False,
+        mini_map=(84, 84, 250),  # buffer length, width
+        rgb_camera=(84, 84),  # buffer length, width
+        depth_camera=(84, 84, True),  # buffer length, width, view_ground
+        show_side_detector=False,
+        show_lane_line_detector=False,
+
+        # NOTE: rgb_clip will be modified by env level config when initialization
+        rgb_clip=True,  # clip 0-255 to 0-1
+        stack_size=3,  # the number of timesteps for stacking image observation
+        rgb_to_grayscale=False,
+        gaussian_noise=0.0,
+        dropout_prob=0.0,
     ),
 
+    # ===== Agent config =====
+    target_vehicle_configs={DEFAULT_AGENT: dict(use_special_color=False, spawn_lane_index=None)},
+
     # ===== Engine Core config =====
-    window_size=(1200, 900),  # width, height
+    window_size=(1200, 900),  # or (width, height), if set to None, it will be automatically determined
     physics_world_step_size=2e-2,
     show_fps=True,
     global_light=False,
@@ -95,14 +122,34 @@ BASE_DEFAULT_CONFIG = dict(
     headless_machine_render=False,
     # turn on to profile the efficiency
     pstats=False,
+    # if need running in offscreen
+    offscreen_render=False,
+    # accelerate the lidar perception
+    _disable_detector_mask=False,
+    # clip rgb to (0, 1)
+    rgb_clip=True,
+    # None: unlimited, number: fps
+    force_render_fps=None,
+    # if set to True all objects will be force destroyed when call clear()
+    force_destroy=False,
 
     # ===== Others =====
     # The maximum distance used in PGLOD. Set to None will use the default values.
     max_distance=None,
     # Force to generate objects in the left lane.
     _debug_crash_object=False,
-    record_episode=False,
-    horizon=None,  # The maximum length of each episode. Set to None to remove this constraint
+    horizon=None,  # The maximum length of each environmental episode. Set to None to remove this constraint
+    max_step_per_agent=None,  # The maximum length of each agent episode. Raise max_step termination when reaches.
+    show_interface_navi_mark=True,
+    show_mouse=True,
+    show_skybox=True,
+    show_terrain=True,
+    show_interface=True,
+
+    # record/replay metadata
+    record_episode=False,  # when replay_episode is not None ,this option will be useless
+    replay_episode=None,  # set the replay file to enable replay
+    only_reset_when_replay=False,  # Scenario will only be initialized, while future trajectories will not be replayed
 )
 
 
@@ -117,6 +164,8 @@ class BaseEnv(gym.Env):
 
     # ===== Intialization =====
     def __init__(self, config: dict = None):
+        if config is None:
+            config = {}
         merged_config = self._merge_extra_config(config)
         global_config = self._post_process_config(merged_config)
         self.config = global_config
@@ -133,15 +182,8 @@ class BaseEnv(gym.Env):
             init_observations=self._get_observations(), init_action_space=self._get_action_space()
         )
 
-        # map setting
-        self.start_seed = self.config["start_seed"]
-        self.env_num = self.config["environment_num"]
-
         # lazy initialization, create the main vehicle in the lazy_init() func
         self.engine: Optional[BaseEngine] = None
-        self._top_down_renderer = None
-        self.episode_steps = 0
-        # self.current_seed = None
 
         # In MARL envs with respawn mechanism, varying episode lengths might happen.
         self.dones = None
@@ -154,6 +196,8 @@ class BaseEnv(gym.Env):
 
     def _post_process_config(self, config):
         """Add more special process to merged config"""
+        config["vehicle_config"]["random_agent_model"] = config["random_agent_model"]
+        config["vehicle_config"]["rgb_clip"] = config["rgb_clip"]
         return config
 
     def _get_observations(self) -> Dict[str, "ObservationBase"]:
@@ -198,10 +242,9 @@ class BaseEnv(gym.Env):
 
     # ===== Run-time =====
     def step(self, actions: Union[np.ndarray, Dict[AnyStr, np.ndarray]]):
-        self.episode_steps += 1
         actions = self._preprocess_actions(actions)
-        step_infos = self._step_simulator(actions)
-        o, r, d, i = self._get_step_return(actions, step_infos)
+        engine_info = self._step_simulator(actions)
+        o, r, d, i = self._get_step_return(actions, engine_info=engine_info)
         return o, r, d, i
 
     def _preprocess_actions(self, actions: Union[np.ndarray, Dict[AnyStr, np.ndarray]]) \
@@ -256,9 +299,11 @@ class BaseEnv(gym.Env):
         :param text:text to show
         :return: when mode is 'rgb', image array is returned
         """
-        if mode == "top_down":
-            return self._render_topdown(*args, **kwargs)
-        assert self.config["use_render"] or self.engine.mode != RENDER_MODE_NONE, ("render is off now, can not render")
+        if mode in ["top_down", "topdown", "bev", "birdview"]:
+            ret = self._render_topdown(text=text, *args, **kwargs)
+            return ret
+        assert self.config["use_render"] or self.engine.mode != RENDER_MODE_NONE, \
+            ("Panda Renderring is off now, can not render")
         self.engine.render_frame(text)
         if mode != "human" and self.config["offscreen_render"]:
             # fetch img from img stack to be make this func compatible with other render func in RL setting
@@ -278,24 +323,30 @@ class BaseEnv(gym.Env):
         # logging.warning("You do not set 'offscreen_render' or 'offscreen_render' to True, so no image will be returned!")
         return None
 
-    def reset(self, episode_data: dict = None, force_seed: Union[None, int] = None):
+    def reset(self, force_seed: Union[None, int] = None):
         """
         Reset the env, scene can be restored and replayed by giving episode_data
         Reset the environment or load an episode from episode data to recover is
-        :param episode_data: Feed the episode data to replay an episode
         :param force_seed: The seed to set the env.
         :return: None
         """
         self.lazy_init()  # it only works the first time when reset() is called to avoid the error when render
         self._reset_global_seed(force_seed)
+        if self.engine is None:
+            raise ValueError(
+                "Current MetaDrive instance is broken. Please make sure there is only one active MetaDrive "
+                "environment exists in one process. You can try to call env.close() and then call "
+                "env.reset() to rescue this environment. However, a better and safer solution is to check the "
+                "singleton of MetaDrive and restart your program."
+            )
         self.engine.reset()
         if self._top_down_renderer is not None:
             self._top_down_renderer.reset(self.current_map)
 
         self.dones = {agent_id: False for agent_id in self.vehicles.keys()}
-        self.episode_steps = 0
         self.episode_rewards = defaultdict(float)
         self.episode_lengths = defaultdict(int)
+
         assert (len(self.vehicles) == self.num_agents) or (self.num_agents == -1)
 
         return self._get_reset_return()
@@ -308,7 +359,7 @@ class BaseEnv(gym.Env):
             ret[v_id] = self.observations[v_id].observe(v)
         return ret if self.is_multi_agent else self._wrap_as_single_agent(ret)
 
-    def _get_step_return(self, actions, step_infos):
+    def _get_step_return(self, actions, engine_info):
         # update obs, dones, rewards, costs, calculate done at first !
         obses = {}
         done_infos = {}
@@ -316,25 +367,21 @@ class BaseEnv(gym.Env):
         reward_infos = {}
         rewards = {}
         for v_id, v in self.vehicles.items():
-            obses[v_id] = self.observations[v_id].observe(v)
-            done_function_result, done_infos[v_id] = self.done_function(v_id)
             rewards[v_id], reward_infos[v_id] = self.reward_function(v_id)
+            done_function_result, done_infos[v_id] = self.done_function(v_id)
             _, cost_infos[v_id] = self.cost_function(v_id)
             done = done_function_result or self.dones[v_id]
             self.dones[v_id] = done
+            o = self.observations[v_id].observe(v)
+            obses[v_id] = o
 
-        should_done = self.config["horizon"] and self.episode_steps >= self.config["horizon"]
-        termination_infos = self.for_each_vehicle(auto_termination, should_done)
+        step_infos = concat_step_infos([engine_info, done_infos, reward_infos, cost_infos])
 
-        step_infos = concat_step_infos([
-            step_infos,
-            done_infos,
-            reward_infos,
-            cost_infos,
-            termination_infos,
-        ])
-
-        if should_done:
+        # For extreme case only. Force to terminate all vehicles if the environmental step exceeds 5 times horizon.
+        should_external_done = False
+        if self.config["horizon"] is not None:
+            should_external_done = self.episode_step > 5 * self.config["horizon"]
+        if should_external_done:
             for k in self.dones:
                 self.dones[k] = True
 
@@ -344,6 +391,7 @@ class BaseEnv(gym.Env):
             step_infos[v_id]["episode_reward"] = self.episode_rewards[v_id]
             self.episode_lengths[v_id] += 1
             step_infos[v_id]["episode_length"] = self.episode_lengths[v_id]
+
         if not self.is_multi_agent:
             return self._wrap_as_single_agent(obses), self._wrap_as_single_agent(rewards), \
                    self._wrap_as_single_agent(dones), self._wrap_as_single_agent(step_infos)
@@ -353,8 +401,6 @@ class BaseEnv(gym.Env):
     def close(self):
         if self.engine is not None:
             close_engine()
-        if self._top_down_renderer is not None:
-            self._top_down_renderer.close()
 
     def force_close(self):
         print("Closing environment ... Please wait")
@@ -380,8 +426,12 @@ class BaseEnv(gym.Env):
         ego_v = self.vehicles[DEFAULT_AGENT]
         return ego_v
 
-    def get_single_observation(self, vehicle_config: "Config") -> "ObservationBase":
-        raise NotImplementedError()
+    def get_single_observation(self, vehicle_config: "Config"):
+        if self.config["offscreen_render"]:
+            o = ImageStateObservation(vehicle_config)
+        else:
+            o = LidarStateObservation(vehicle_config)
+        return o
 
     def _wrap_as_single_agent(self, data):
         return data[next(iter(self.vehicles.keys()))]
@@ -409,7 +459,7 @@ class BaseEnv(gym.Env):
         :return: Dict
         """
         ret = self.agent_manager.get_observation_spaces()
-        if not self.is_multi_agent:
+        if (not self.is_multi_agent) or self.config["return_single_space"]:
             return next(iter(ret.values()))
         else:
             return gym.spaces.Dict(ret)
@@ -421,7 +471,7 @@ class BaseEnv(gym.Env):
         :return: Dict
         """
         ret = self.agent_manager.get_action_spaces()
-        if not self.is_multi_agent:
+        if (not self.is_multi_agent) or self.config["return_single_space"]:
             return next(iter(ret.values()))
         else:
             return gym.spaces.Dict(ret)
@@ -434,37 +484,40 @@ class BaseEnv(gym.Env):
         """
         return self.agent_manager.active_agents
 
+    @property
+    def vehicles_including_just_terminated(self):
+        """
+        Return all vehicles that occupy some space in current environments
+        :return: Dict[agent_id:vehicle]
+        """
+        ret = self.agent_manager.active_agents
+        ret.update(self.agent_manager.just_terminated_agents)
+        return ret
+
     def setup_engine(self):
         """
         Engine setting after launching
         """
         self.engine.accept("r", self.reset)
-        self.engine.accept("escape", sys.exit)
         self.engine.accept("p", self.capture)
         self.engine.register_manager("agent_manager", self.agent_manager)
+        self.engine.register_manager("record_manager", RecordManager())
+        self.engine.register_manager("replay_manager", ReplayManager())
 
     @property
     def current_map(self):
-        return self.engine.map_manager.current_map
+        return self.engine.current_map
 
-    def _reset_global_seed(self, force_seed):
-        # create map
-        if force_seed is not None:
-            current_seed = force_seed
-        else:
-            current_seed = get_np_random(self._DEBUG_RANDOM_SEED
-                                         ).randint(self.start_seed, self.start_seed + self.env_num)
+    def _reset_global_seed(self, force_seed=None):
+        current_seed = force_seed if force_seed is not None else get_np_random(None).randint(0, int(1e4))
         self.seed(current_seed)
 
     @property
     def maps(self):
-        return self.engine.map_manager.pg_maps
+        return self.engine.map_manager.maps
 
-    def _render_topdown(self, *args, **kwargs):
-        if self._top_down_renderer is None:
-            from metadrive.obs.top_down_renderer import TopDownRenderer
-            self._top_down_renderer = TopDownRenderer(self, self.current_map, *args, **kwargs)
-        return self._top_down_renderer.render(self.agent_manager)
+    def _render_topdown(self, text, *args, **kwargs):
+        return self.engine.render_topdown(text, *args, **kwargs)
 
     @property
     def main_camera(self):
@@ -473,3 +526,11 @@ class BaseEnv(gym.Env):
     @property
     def current_track_vehicle(self):
         return self.engine.current_track_vehicle
+
+    @property
+    def _top_down_renderer(self):
+        return self.engine._top_down_renderer
+
+    @property
+    def episode_step(self):
+        return self.engine.episode_step if self.engine is not None else 0
