@@ -1,17 +1,15 @@
-from metadrive.component.blocks.bottleneck import Merge, Split
-from metadrive.component.blocks.first_block import FirstPGBlock
 from metadrive.component.map.pg_map import PGMap
-from metadrive.component.road.road import Road
-from metadrive.envs.marl_envs.marl_inout_roundabout import LidarStateObservationMARound
+from metadrive.component.pgblock.bottleneck import Merge, Split
+from metadrive.component.pgblock.first_block import FirstPGBlock
+from metadrive.component.road_network import Road
 from metadrive.envs.marl_envs.multi_agent_metadrive import MultiAgentMetaDrive
-from metadrive.obs.observation_base import ObservationBase
+from metadrive.manager.map_manager import PGMapManager
 from metadrive.utils import Config
-from metadrive.manager.map_manager import MapManager
 from metadrive.utils.math_utils import clip
 
 MABottleneckConfig = dict(
     spawn_roads=[Road(FirstPGBlock.NODE_2, FirstPGBlock.NODE_3), -Road(Split.node(2, 0, 0), Split.node(2, 0, 1))],
-    num_agents=12,
+    num_agents=20,
     map_config=dict(exit_length=60, bottle_lane_num=4, neck_lane_num=1, neck_length=20),
     top_down_camera_initial_x=95,
     top_down_camera_initial_y=15,
@@ -68,7 +66,7 @@ class MABottleneckMap(PGMap):
         self.blocks.append(split)
 
 
-class MABottleneckMapManager(MapManager):
+class MABottleneckPGMapManager(PGMapManager):
     def reset(self):
         config = self.engine.global_config
         if len(self.spawned_objects) == 0:
@@ -88,9 +86,6 @@ class MultiAgentBottleneckEnv(MultiAgentMetaDrive):
         MABottleneckConfig["map_config"]["lane_num"] = MABottleneckConfig["map_config"]["bottle_lane_num"]
         return MultiAgentMetaDrive.default_config().update(MABottleneckConfig, allow_add_new_key=True)
 
-    def get_single_observation(self, vehicle_config: "Config") -> "ObservationBase":
-        return LidarStateObservationMARound(vehicle_config)
-
     def reward_function(self, vehicle_id: str):
         """
         Override this func to get a new reward function
@@ -105,12 +100,12 @@ class MultiAgentBottleneckEnv(MultiAgentMetaDrive):
             current_lane = vehicle.lane
         else:
             current_lane = vehicle.navigation.current_ref_lanes[0]
-            current_road = vehicle.current_road
+            current_road = vehicle.navigation.current_road
         long_last, _ = current_lane.local_coordinates(vehicle.last_position)
         long_now, lateral_now = current_lane.local_coordinates(vehicle.position)
 
         # reward for lane keeping, without it vehicle can learn to overtake but fail to keep in lane
-        if self.config["use_lateral"]:
+        if self.config["use_lateral_reward"]:
             lateral_factor = clip(1 - 2 * abs(lateral_now) / vehicle.navigation.get_current_lane_width(), 0.0, 1.0)
         else:
             lateral_factor = 1.0
@@ -141,7 +136,7 @@ class MultiAgentBottleneckEnv(MultiAgentMetaDrive):
 
     def setup_engine(self):
         super(MultiAgentBottleneckEnv, self).setup_engine()
-        self.engine.update_manager("map_manager", MABottleneckMapManager())
+        self.engine.update_manager("map_manager", MABottleneckPGMapManager())
 
 
 def _draw():
