@@ -1,14 +1,13 @@
 import copy
-from metadrive.manager.spawn_manager import SpawnManager
-from metadrive.manager.map_manager import MapManager
-from metadrive.component.blocks.first_block import FirstPGBlock
-from metadrive.component.blocks.intersection import InterSection
+
 from metadrive.component.map.pg_map import PGMap
-from metadrive.component.road.road import Road
-from metadrive.envs.marl_envs.marl_inout_roundabout import LidarStateObservationMARound
+from metadrive.component.pgblock.first_block import FirstPGBlock
+from metadrive.component.pgblock.intersection import InterSection
+from metadrive.component.road_network import Road
 from metadrive.envs.marl_envs.multi_agent_metadrive import MultiAgentMetaDrive
-from metadrive.obs.observation_base import ObservationBase
-from metadrive.utils import get_np_random, Config
+from metadrive.manager.map_manager import PGMapManager
+from metadrive.manager.spawn_manager import SpawnManager
+from metadrive.utils import Config
 
 MAIntersectionConfig = dict(
     spawn_roads=[
@@ -17,7 +16,7 @@ MAIntersectionConfig = dict(
         -Road(InterSection.node(1, 1, 0), InterSection.node(1, 1, 1)),
         -Road(InterSection.node(1, 2, 0), InterSection.node(1, 2, 1)),
     ],
-    num_agents=12,
+    num_agents=30,
     map_config=dict(exit_length=60, lane_num=2),
     top_down_camera_initial_x=80,
     top_down_camera_initial_y=0,
@@ -45,23 +44,45 @@ class MAIntersectionMap(PGMap):
 
         # Build Intersection
         InterSection.EXIT_PART_LENGTH = length
+
+        if "radius" in self.config and self.config["radius"]:
+            extra_kwargs = dict(radius=self.config["radius"])
+        else:
+            extra_kwargs = {}
         last_block = InterSection(
-            1, last_block.get_socket(index=0), self.road_network, random_seed=1, ignore_intersection_checking=False
+            1,
+            last_block.get_socket(index=0),
+            self.road_network,
+            random_seed=1,
+            ignore_intersection_checking=False,
+            **extra_kwargs
         )
-        last_block.add_u_turn(True)
+
+        if self.config["lane_num"] > 1:
+            # We disable U turn in TinyInter environment!
+            last_block.enable_u_turn(True)
+        else:
+            last_block.enable_u_turn(False)
+
         last_block.construct_block(parent_node_path, physics_world)
         self.blocks.append(last_block)
 
 
 class MAIntersectionSpawnManager(SpawnManager):
+    def __init__(self, disable_u_turn=False):
+        super(MAIntersectionSpawnManager, self).__init__()
+        self.disable_u_turn = disable_u_turn
+
     def update_destination_for(self, agent_id, vehicle_config):
         end_roads = copy.deepcopy(self.engine.global_config["spawn_roads"])
+        if self.disable_u_turn:  # Remove the spawn road from end roads
+            end_roads = [r for r in end_roads if Road(*vehicle_config["spawn_lane_index"][:2]) != r]
         end_road = -self.np_random.choice(end_roads)  # Use negative road!
-        vehicle_config["destination_node"] = end_road.end_node
+        vehicle_config["destination"] = end_road.end_node
         return vehicle_config
 
 
-class MAIntersectionMapManager(MapManager):
+class MAIntersectionPGMapManager(PGMapManager):
     def reset(self):
         config = self.engine.global_config
         if len(self.spawned_objects) == 0:
@@ -78,13 +99,11 @@ class MultiAgentIntersectionEnv(MultiAgentMetaDrive):
     def default_config() -> Config:
         return MultiAgentMetaDrive.default_config().update(MAIntersectionConfig, allow_add_new_key=True)
 
-    def get_single_observation(self, vehicle_config: "Config") -> "ObservationBase":
-        return LidarStateObservationMARound(vehicle_config)
-
     def setup_engine(self):
+        disable_u_turn = self.config["map_config"]["lane_num"] < 2
         super(MultiAgentIntersectionEnv, self).setup_engine()
-        self.engine.update_manager("map_manager", MAIntersectionMapManager())
-        self.engine.update_manager("spawn_manager", MAIntersectionSpawnManager())
+        self.engine.update_manager("map_manager", MAIntersectionPGMapManager())
+        self.engine.update_manager("spawn_manager", MAIntersectionSpawnManager(disable_u_turn=disable_u_turn))
 
 
 def _draw():
@@ -210,7 +229,7 @@ def _vis():
             "debug": True,
             "allow_respawn": False,
             "manual_control": True,
-            "num_agents": 2,
+            "num_agents": 20,
             "delay_done": 2,
         }
     )
