@@ -1,20 +1,20 @@
 import gym
 import numpy as np
 
-from metadrive.component.blocks.bottleneck import Merge, Split
-from metadrive.component.blocks.first_block import FirstPGBlock
-from metadrive.component.blocks.tollgate import TollGate
 from metadrive.component.map.pg_map import PGMap
-from metadrive.component.road.road import Road
+from metadrive.component.pgblock.bottleneck import Merge, Split
+from metadrive.component.pgblock.first_block import FirstPGBlock
+from metadrive.component.pgblock.tollgate import TollGate
+from metadrive.component.road_network import Road
 from metadrive.constants import TerminationState
-from metadrive.envs.marl_envs.multi_agent_metadrive import MultiAgentMetaDrive, pygame_replay
+from metadrive.envs.marl_envs.multi_agent_metadrive import MultiAgentMetaDrive
+from metadrive.manager.map_manager import PGMapManager
 from metadrive.obs.state_obs import LidarStateObservation, StateObservation
 from metadrive.utils import Config, clip
-from metadrive.manager.map_manager import MapManager
 
 MATollConfig = dict(
     spawn_roads=[Road(FirstPGBlock.NODE_2, FirstPGBlock.NODE_3), -Road(Merge.node(3, 0, 0), Merge.node(3, 0, 1))],
-    num_agents=12,
+    num_agents=40,
     map_config=dict(exit_length=70, lane_num=3, toll_lane_num=8, toll_length=10),
     top_down_camera_initial_x=125,
     top_down_camera_initial_y=0,
@@ -48,7 +48,7 @@ class StayTimeManager:
 
     def record(self, agents, time_step):
         for v_id, v in agents.items():
-            cur_block_id = v.current_road.block_ID()
+            cur_block_id = v.navigation.current_road.block_ID()
             if v_id in self.last_block:
                 last_block_id = self.last_block[v_id]
                 self.last_block[v_id] = cur_block_id
@@ -93,7 +93,7 @@ class TollGateObservation(LidarStateObservation):
         self.in_toll_time = 0
 
     def observe(self, vehicle):
-        cur_block_is_toll = vehicle.current_road.block_ID() == TollGate.ID
+        cur_block_is_toll = vehicle.navigation.current_road.block_ID() == TollGate.ID
         self.in_toll_time += 1 if cur_block_is_toll else 0
         if not cur_block_is_toll:
             toll_obs = [0.0, 0.0]
@@ -104,7 +104,8 @@ class TollGateObservation(LidarStateObservation):
         # print(toll_obs)
         state = self.state_observe(vehicle)
         other_v_info = self.lidar_observe(vehicle)
-        return np.concatenate((state, np.asarray(other_v_info), np.asarray(toll_obs)))
+        ret = np.concatenate((state, np.asarray(other_v_info), np.asarray(toll_obs)))
+        return ret.astype(np.float32)
 
 
 class MATollGateMap(PGMap):
@@ -159,7 +160,7 @@ class MATollGateMap(PGMap):
         self.blocks.append(merge)
 
 
-class MATollGateMapManager(MapManager):
+class MATollGatePGMapManager(PGMapManager):
     def reset(self):
         config = self.engine.global_config
         if len(self.spawned_objects) == 0:
@@ -172,7 +173,7 @@ class MATollGateMapManager(MapManager):
 
 
 class MultiAgentTollgateEnv(MultiAgentMetaDrive):
-    def __init__(self, config):
+    def __init__(self, config=None):
         super(MultiAgentTollgateEnv, self).__init__(config)
         self.stay_time_manager = StayTimeManager()
 
@@ -200,12 +201,12 @@ class MultiAgentTollgateEnv(MultiAgentMetaDrive):
             current_lane = vehicle.lane
         else:
             current_lane = vehicle.navigation.current_ref_lanes[0]
-            current_road = vehicle.current_road
+            current_road = vehicle.navigation.current_road
         long_last, _ = current_lane.local_coordinates(vehicle.last_position)
         long_now, lateral_now = current_lane.local_coordinates(vehicle.position)
 
         # reward for lane keeping, without it vehicle can learn to overtake but fail to keep in lane
-        if self.config["use_lateral"]:
+        if self.config["use_lateral_reward"]:
             lateral_factor = clip(1 - 2 * abs(lateral_now) / vehicle.navigation.get_current_lane_width(), 0.0, 1.0)
         else:
             lateral_factor = 1.0
@@ -213,7 +214,7 @@ class MultiAgentTollgateEnv(MultiAgentMetaDrive):
         reward = 0.0
         reward += self.config["driving_reward"] * (long_now - long_last) * lateral_factor
 
-        if vehicle.current_road.block_ID() == TollGate.ID:
+        if vehicle.navigation.current_road.block_ID() == TollGate.ID:
             if vehicle.overspeed:  # Too fast!
                 reward = -self.config["overspeed_penalty"] * vehicle.speed / vehicle.max_speed
             else:
@@ -272,12 +273,12 @@ class MultiAgentTollgateEnv(MultiAgentMetaDrive):
 
     def step(self, actions):
         o, r, d, i = super(MultiAgentTollgateEnv, self).step(actions)
-        self.stay_time_manager.record(self.agent_manager.active_agents, self.episode_steps)
+        self.stay_time_manager.record(self.agent_manager.active_agents, self.episode_step)
         return o, r, d, i
 
     def setup_engine(self):
         super(MultiAgentTollgateEnv, self).setup_engine()
-        self.engine.update_manager("map_manager", MATollGateMapManager())
+        self.engine.update_manager("map_manager", MATollGatePGMapManager())
 
 
 def _draw():
@@ -406,7 +407,7 @@ def _vis():
             "use_render": True,
             # "debug": True,
             "manual_control": True,
-            "num_agents": 1,
+            "num_agents": 18,
         }
     )
     o = env.reset()
@@ -432,7 +433,7 @@ def _vis():
         render_text["dist_to_left"] = env.current_track_vehicle.dist_to_left_side
         render_text["overspeed"] = env.current_track_vehicle.overspeed
         render_text["lane"] = env.current_track_vehicle.lane_index
-        render_text["block"] = env.current_track_vehicle.current_road.block_ID()
+        render_text["block"] = env.current_track_vehicle.navigation.current_road.block_ID()
         env.render(text=render_text)
         if d["__all__"]:
             print(info)
