@@ -20,6 +20,7 @@ from metadrive import (
     MultiAgentRoundaboutEnv, MultiAgentParkingLotEnv
 )
 from metadrive.constants import HELP_MESSAGE
+from metadrive.policy.idm_policy import ManualControllableIDMPolicy
 
 if __name__ == "__main__":
     envs = dict(
@@ -50,7 +51,7 @@ if __name__ == "__main__":
             "use_render": True if not args.pygame_render else False,
             "manual_control": True,
             "crash_done": False,
-            "IDM_agent": True
+            "agent_policy": ManualControllableIDMPolicy
         }
     )
     try:
