@@ -1,10 +1,23 @@
 from typing import List, Tuple, Union
+import logging
+import math
+from metadrive.component.road_network.edge_road_network import EdgeRoadNetwork
+
+import numpy as np
+from metadrive.component.block.base_block import BaseBlock
+from metadrive.component.lane.waymo_lane import WaymoLane
+from metadrive.constants import DrivableAreaProperty
+from metadrive.constants import LineType, LineColor
+from metadrive.constants import WaymoLaneProperty
+from metadrive.utils.interpolating_line import InterpolatingLine
+from metadrive.utils.math_utils import wrap_to_pi, norm
+from metadrive.utils.waymo_utils.waymo_utils import RoadLineType, RoadEdgeType, convert_polyline_to_metadrive
 
 import numpy as np
 
 from metadrive.component.lane.circular_lane import CircularLane
 from metadrive.component.lane.straight_lane import StraightLane
-from metadrive.constants import LineType
+from metadrive.constants import LineType, LineColor
 from metadrive.utils.utils import import_pygame
 
 PositionType = Union[Tuple[float, float], np.ndarray]
@@ -212,8 +225,9 @@ class VehicleGraphics:
         angle = np.rad2deg(heading)
         box = [pygame.math.Vector2(p) for p in [(-h / 2, -w / 2), (-h / 2, w / 2), (h / 2, w / 2), (h / 2, -w / 2)]]
         box_rotate = [p.rotate(angle) + position for p in box]
-        pygame.draw.polygon(surface, color=color, points=box_rotate)
-        if draw_countour:
+
+        pygame.draw.polygon(surface, color, box_rotate)
+        if draw_countour and pygame.ver.startswith("2"):
             pygame.draw.polygon(surface, cls.BLACK, box_rotate, width=contour_width)  # , 1)
 
         # Label
@@ -248,7 +262,7 @@ class LaneGraphics:
     LANE_LINE_WIDTH: float = 1
 
     @classmethod
-    def display(cls, lane, surface, two_side=True, color=None) -> None:
+    def display(cls, lane, surface, two_side=True, use_line_color=False, color=None) -> None:
         """
         Display a lane on a surface.
 
@@ -261,6 +275,13 @@ class LaneGraphics:
         s_origin, _ = lane.local_coordinates(surface.origin)
         s0 = (int(s_origin) // cls.STRIPE_SPACING - stripes_count // 2) * cls.STRIPE_SPACING
         for side in range(side):
+            if use_line_color:
+                if lane.line_colors[side] == LineColor.YELLOW and lane.line_types[side] == LineType.CONTINUOUS:
+                    color = (0, 80, 220)
+                elif lane.line_types[side] == LineType.SIDE:
+                    color = (160, 160, 160)
+                else:
+                    color = (80, 80, 80)
             if lane.line_types[side] == LineType.BROKEN:
                 cls.striped_line(lane, surface, stripes_count, s0, side, color=color)
             # circular side or continuous, it is same now
@@ -279,6 +300,43 @@ class LaneGraphics:
             else:
                 raise ValueError("I don't know how to draw this line type: {}".format(lane.line_types[side]))
 
+    @classmethod
+    def display_waymo(cls, waymo_poly_line, type, surface) -> None:
+        """
+        Display a lane on a surface.
+
+        :param lane: the lane to be displayed
+        :param surface: the pygame surface
+        :param two_side: draw two sides of the lane, or only one side
+        """
+        lane = waymo_poly_line
+        if RoadLineType.is_yellow(type):
+            color = (0, 80, 220)
+        elif RoadEdgeType.is_road_edge(type):
+            color = (160, 160, 160)
+        else:
+            color = (80, 80, 80)
+        if RoadLineType.is_road_line(type) or RoadEdgeType.is_road_edge(type):
+            if len(waymo_poly_line.segment_property) <= 1:
+                return
+            stripes_count = int(
+                2 * (surface.get_height() + surface.get_width()) / (cls.STRIPE_SPACING * surface.scaling)
+            )
+            s_origin, _ = lane.local_coordinates(surface.origin)
+            s0 = (int(s_origin) // cls.STRIPE_SPACING - stripes_count // 2) * cls.STRIPE_SPACING
+
+            if RoadLineType.is_broken(type):
+                starts = s0 + np.arange(stripes_count) * cls.STRIPE_SPACING
+                ends = s0 + np.arange(stripes_count) * cls.STRIPE_SPACING + cls.STRIPE_LENGTH
+                lats = [0 for s in starts]
+            else:
+                starts = s0 + np.arange(stripes_count) * cls.STRIPE_SPACING
+                ends = s0 + np.arange(stripes_count) * cls.STRIPE_SPACING + cls.STRIPE_SPACING
+                lats = [0 for s in starts]
+            cls.draw_stripes(lane, surface, starts, ends, lats, color=color)
+        elif type == "center_lane" or type is None:
+            pass
+
     @classmethod
     def striped_line(cls, lane, surface, stripes_count: int, longitudinal: float, side: int, color=None) -> None:
         """
@@ -352,14 +410,14 @@ class LaneGraphics:
 
     @classmethod
     def simple_draw(cls, lane, surface, color=(255, 255, 255)):
-        from metadrive.component.blocks.pg_block import PGBlock
-        segment_num = int(lane.length / PGBlock.CIRCULAR_SEGMENT_LENGTH)
+        from metadrive.component.pgblock.pg_block import PGBlock
+        segment_num = int(lane.length / PGBlock.LANE_SEGMENT_LENGTH)
         width = lane.width
         for segment in range(segment_num):
-            p_1 = lane.position(segment * PGBlock.CIRCULAR_SEGMENT_LENGTH, -width / 2)
-            p_2 = lane.position(segment * PGBlock.CIRCULAR_SEGMENT_LENGTH, width / 2)
-            p_3 = lane.position((segment + 1) * PGBlock.CIRCULAR_SEGMENT_LENGTH, width / 2)
-            p_4 = lane.position((segment + 1) * PGBlock.CIRCULAR_SEGMENT_LENGTH, -width / 2)
+            p_1 = lane.position(segment * PGBlock.LANE_SEGMENT_LENGTH, -width / 2)
+            p_2 = lane.position(segment * PGBlock.LANE_SEGMENT_LENGTH, width / 2)
+            p_3 = lane.position((segment + 1) * PGBlock.LANE_SEGMENT_LENGTH, width / 2)
+            p_4 = lane.position((segment + 1) * PGBlock.LANE_SEGMENT_LENGTH, -width / 2)
             pygame.draw.polygon(
                 surface, color,
                 [surface.pos2pix(*p_1),
@@ -369,8 +427,8 @@ class LaneGraphics:
             )
 
         # # for last part
-        p_1 = lane.position(segment_num * PGBlock.CIRCULAR_SEGMENT_LENGTH, -width / 2)
-        p_2 = lane.position(segment_num * PGBlock.CIRCULAR_SEGMENT_LENGTH, width / 2)
+        p_1 = lane.position(segment_num * PGBlock.LANE_SEGMENT_LENGTH, -width / 2)
+        p_2 = lane.position(segment_num * PGBlock.LANE_SEGMENT_LENGTH, width / 2)
         p_3 = lane.position(lane.length, width / 2)
         p_4 = lane.position(lane.length, -width / 2)
         pygame.draw.polygon(
@@ -440,7 +498,8 @@ class ObservationWindowMultiChannel:
             mask = pygame.mask.from_threshold(ret[key], (0, 0, 0, 0), (10, 10, 10, 255))
             mask.to_surface(canvas, setcolor=None, unsetcolor=color)
 
-        _draw(canvas, "navigation", pygame.Color("Blue"))
+        if "navigation" in ret:
+            _draw(canvas, "navigation", pygame.Color("Blue"))
         _draw(canvas, "road_network", pygame.Color("White"))
         _draw(canvas, "traffic_flow", pygame.Color("Red"))
         _draw(canvas, "target_vehicle", pygame.Color("Green"))
