@@ -1,5 +1,7 @@
-from metadrive.base_class.randomizable import Randomizable
+import logging
+
 from metadrive.base_class.configurable import Configurable
+from metadrive.base_class.randomizable import Randomizable
 from metadrive.engine.engine_utils import get_engine
 
 
@@ -28,6 +30,14 @@ class BasePolicy(Randomizable, Configurable):
         pass
 
     def destroy(self):
+        logging.debug("{} is released".format(self.__class__.__name__))
         super(BasePolicy, self).destroy()
         self.control_object = None
         self.engine = None
+
+    @property
+    def name(self):
+        return self.__class__.__name__
+
+    def __repr__(self):
+        return self.name
