@@ -1,12 +1,10 @@
 import logging
 
 import numpy as np
-
 from metadrive.component.vehicle_module.PID_controller import PIDController
 from metadrive.policy.base_policy import BasePolicy
 from metadrive.policy.manual_control_policy import ManualControlPolicy
-from metadrive.utils.math_utils import not_zero, wrap_to_pi, point_distance
-from metadrive.utils.scene_utils import is_same_lane_index, is_following_lane_index
+from metadrive.utils.math_utils import not_zero, wrap_to_pi
 
 
 class FrontBackObjects:
@@ -88,9 +86,9 @@ class FrontBackObjects:
         """
         if ref_lanes is not None:
             assert lane in ref_lanes
-        idx = lane.index[-1]
-        left_lane = ref_lanes[idx - 1] if idx > 0 and ref_lanes is not None else None
-        right_lane = ref_lanes[idx + 1] if ref_lanes and idx + 1 < len(ref_lanes) is not None else None
+        idx = lane.index[-1] if ref_lanes is not None else None
+        left_lane = ref_lanes[idx - 1] if ref_lanes is not None and idx > 0 else None
+        right_lane = ref_lanes[idx + 1] if ref_lanes is not None and idx + 1 < len(ref_lanes) else None
         lanes = [left_lane, lane, right_lane]
 
         min_front_long = [max_distance if lane is not None else None for lane in lanes]
@@ -133,6 +131,42 @@ class FrontBackObjects:
 
         return cls(front_ret, back_ret, min_front_long, min_back_long)
 
+    @classmethod
+    def get_find_front_back_objs_waymo(cls, objs, lane, position, max_distance):
+        """
+        Find objects in front of/behind the lane and its left lanes/right lanes, return objs, dist.
+        If ref_lanes is None, return filter results of this lane
+        """
+        lanes = [None, lane, None]
+
+        min_front_long = [max_distance if lane is not None else None for lane in lanes]
+        min_back_long = [max_distance if lane is not None else None for lane in lanes]
+
+        front_ret = [None, None, None]
+        back_ret = [None, None, None]
+
+        find_front_in_current_lane = [False, False, False]
+        # find_back_in_current_lane = [False, False, False]
+
+        current_long = [lane.local_coordinates(position)[0] if lane is not None else None for lane in lanes]
+
+        for i, lane in enumerate(lanes):
+            if lane is None:
+                continue
+            for obj in objs:
+                if np.linalg.norm(obj.position - position) > max_distance:
+                    continue
+                long, lat = lane.local_coordinates(obj.position)
+                if abs(lat) > lane.width:
+                    continue
+                long = long - current_long[i]
+                if min_front_long[i] > long > 0:
+                    min_front_long[i] = long
+                    front_ret[i] = obj
+                    find_front_in_current_lane[i] = True
+
+        return cls(front_ret, back_ret, min_front_long, min_back_long)
+
 
 class IDMPolicy(BasePolicy):
     """
@@ -184,7 +218,8 @@ class IDMPolicy(BasePolicy):
         self.routing_target_lane = None
         self.available_routing_index_range = None
         self.overtake_timer = self.np_random.randint(0, self.LANE_CHANGE_FREQ)
-
+        self.enable_lane_change = self.engine.global_config.get("enable_idm_lane_change", True)
+        self.disable_idm_deceleration = self.engine.global_config.get("disable_idm_deceleration", False)
         self.heading_pid = PIDController(1.7, 0.01, 3.5)
         self.lateral_pid = PIDController(0.3, .002, 0.05)
 
@@ -193,7 +228,7 @@ class IDMPolicy(BasePolicy):
         sucess = self.move_to_next_road()
         all_objects = self.control_object.lidar.get_surrounding_objects(self.control_object)
         try:
-            if sucess:
+            if sucess and self.enable_lane_change:
                 # perform lane change due to routing
                 acc_front_obj, acc_front_dist, steering_target_lane = self.lane_change_policy(all_objects)
             else:
@@ -212,8 +247,8 @@ class IDMPolicy(BasePolicy):
             acc_front_obj = None
             acc_front_dist = 5
             steering_target_lane = self.routing_target_lane
-            logging.warning("IDM bug! fall back")
-            print("IDM bug! fall back")
+            # logging.warning("IDM bug! fall back")
+            # print("IDM bug! fall back")
 
         # control by PID and IDM
         steering = self.steering_control(steering_target_lane)
@@ -226,9 +261,11 @@ class IDMPolicy(BasePolicy):
         if self.routing_target_lane is None:
             self.routing_target_lane = self.control_object.lane
             return True if self.routing_target_lane in current_lanes else False
+        routing_network = self.control_object.navigation.map.road_network
         if self.routing_target_lane not in current_lanes:
             for lane in current_lanes:
-                if self.routing_target_lane.is_previous_lane_of(lane):
+                if self.routing_target_lane.is_previous_lane_of(lane) or \
+                        routing_network.has_connection(self.routing_target_lane.index, lane.index):
                     # two lanes connect
                     self.routing_target_lane = lane
                     return True
@@ -256,7 +293,7 @@ class IDMPolicy(BasePolicy):
         ego_vehicle = self.control_object
         ego_target_speed = not_zero(self.target_speed, 0)
         acceleration = self.ACC_FACTOR * (1 - np.power(max(ego_vehicle.speed, 0) / ego_target_speed, self.DELTA))
-        if front_obj:
+        if front_obj and (not self.disable_idm_deceleration):
             d = dist_to_front
             speed_diff = self.desired_gap(ego_vehicle, front_obj) / not_zero(d)
             acceleration -= self.ACC_FACTOR * (speed_diff**2)
@@ -357,11 +394,122 @@ class IDMPolicy(BasePolicy):
 class ManualControllableIDMPolicy(IDMPolicy):
     def __init__(self, *args, **kwargs):
         super(ManualControllableIDMPolicy, self).__init__(*args, **kwargs)
-        self.manual_control_policy = ManualControlPolicy()
+        self.manual_control_policy = ManualControlPolicy(*args, **kwargs)
 
     def act(self, agent_id):
-        if self.control_object is self.engine.current_track_vehicle and self.engine.global_config["manual_control"]\
+        if self.control_object is self.engine.current_track_vehicle and self.engine.global_config["manual_control"] \
                 and not self.engine.current_track_vehicle.expert_takeover:
             return self.manual_control_policy.act(agent_id)
         else:
             return super(ManualControllableIDMPolicy, self).act(agent_id)
+
+
+class WaymoIDMPolicy(IDMPolicy):
+    NORMAL_SPEED = 40
+    WAYMO_IDM_MAX_DIST = 20
+
+    # DEACC_FACTOR = -20
+    # ACC_FACTOR = 0.5
+    # TIME_WANTED = 80
+    # DISTANCE_WANTED = 55.0
+
+    def __init__(self, control_object, random_seed, traj_to_follow, policy_index):
+        super(WaymoIDMPolicy, self).__init__(control_object=control_object, random_seed=random_seed)
+        self.policy_index = policy_index
+        self.traj_to_follow = traj_to_follow
+        self.target_speed = self.NORMAL_SPEED
+        self.routing_target_lane = self.traj_to_follow
+        self.available_routing_index_range = None
+        self.overtake_timer = self.np_random.randint(0, self.LANE_CHANGE_FREQ)
+        self.enable_lane_change = False
+
+        self.heading_pid = PIDController(1.2, 0.1, 3.5)
+        self.lateral_pid = PIDController(0.3, .0, 0.0)
+
+        self.last_action = [0, 0]
+
+    def steering_control(self, target_lane) -> float:
+        # heading control following a lateral distance control
+        ego_vehicle = self.control_object
+        long, lat = target_lane.local_coordinates(ego_vehicle.position)
+        lane_heading = target_lane.heading_theta_at(long + 1)
+        v_heading = ego_vehicle.heading_theta
+        steering = self.heading_pid.get_result(wrap_to_pi(lane_heading - v_heading))
+        # steering += self.lateral_pid.get_result(-lat)
+        return float(steering)
+
+    def act(self, do_speed_control, *args, **kwargs):
+        # concat lane
+        try:
+            if do_speed_control:
+                all_objects = self.control_object.lidar.get_surrounding_objects(self.control_object)
+                # can not find routing target lane
+                surrounding_objects = FrontBackObjects.get_find_front_back_objs_waymo(
+                    all_objects,
+                    self.routing_target_lane,
+                    self.control_object.position,
+                    max_distance=self.WAYMO_IDM_MAX_DIST
+                )
+                acc_front_obj = surrounding_objects.front_object()
+                acc_front_dist = surrounding_objects.front_min_distance()
+
+                acc = self.acceleration(acc_front_obj, acc_front_dist)
+            else:
+                acc = self.last_action[-1]
+        except:
+            acc = 0
+            print("WaymoIDM Longitudinal Planning failed, acceleration fall back to 0")
+
+        # if self.policy_index % 2 == 0:
+        steering_target_lane = self.routing_target_lane
+        # control by PID and IDM
+        steering = self.steering_control(steering_target_lane)
+        # else:
+        #     steering = self.last_action[0]
+        self.last_action = [steering, acc]
+        return [steering, acc]
+
+
+class EgoWaymoIDMPolicy(IDMPolicy):
+    """
+    This policy is for Vehicles using navigation module, especially, ego car
+    """
+    NORMAL_SPEED = 30
+
+    def __init__(self, control_object, random_seed):
+        super(EgoWaymoIDMPolicy, self).__init__(control_object=control_object, random_seed=random_seed)
+        self.target_speed = self.NORMAL_SPEED
+        self.routing_target_lane = None
+        self.available_routing_index_range = None
+        self.overtake_timer = self.np_random.randint(0, self.LANE_CHANGE_FREQ)
+        self.enable_lane_change = False
+
+        self.heading_pid = PIDController(1.7, 0.01, 3.5)
+        self.lateral_pid = PIDController(0.3, .0, 0.0)
+
+    def steering_control(self, target_lane) -> float:
+        # heading control following a lateral distance control
+        ego_vehicle = self.control_object
+        long, lat = target_lane.local_coordinates(ego_vehicle.position)
+        lane_heading = target_lane.heading_theta_at(long + 1)
+        v_heading = ego_vehicle.heading_theta
+        steering = self.heading_pid.get_result(wrap_to_pi(lane_heading - v_heading))
+        # steering += self.lateral_pid.get_result(-lat)
+        return float(steering)
+
+    def move_to_next_road(self):
+        # routing target lane is in current ref lanes
+        current_lanes = self.control_object.navigation.current_ref_lanes
+        if self.routing_target_lane is None:
+            self.routing_target_lane = self.control_object.lane
+            return True if self.routing_target_lane in current_lanes else False
+        if self.routing_target_lane not in current_lanes:
+            for lane in current_lanes:
+                self.routing_target_lane = lane
+                return True
+                # lane change for lane num change
+            self.routing_target_lane = self.control_object.navigation.map.road_network.get_lane(
+                self.control_object.navigation.next_checkpoint_lane_index
+            )
+            return True
+        return False
