@@ -1,9 +1,8 @@
-from metadrive.envs.generation_envs.remote_metadrive_env import RemoteMetaDrive
-
 # This test is broken for some reasons. Just remove it from CI temporarily.
 
 
 def _test_remote_metadrive_env():
+    from metadrive.envs.remote_env import RemoteMetaDrive
     # Test
     envs = [RemoteMetaDrive(dict(map=7)) for _ in range(3)]
     ret = [env.reset() for env in envs]
