@@ -139,7 +139,8 @@ def test_ma_roundabout_horizon():
                         assert i[kkk][TerminationState.OUT_OF_ROAD]
 
                 for kkk, iii in i.items():
-                    if iii and (iii[TerminationState.OUT_OF_ROAD] or iii["cost"] == 778):
+                    if (TerminationState.OUT_OF_ROAD in iii and iii[TerminationState.OUT_OF_ROAD]) or \
+                            ("cost" in iii and iii["cost"] == 778):
                         assert d[kkk]
                         assert i[kkk]["cost"] == 778
                         assert i[kkk][TerminationState.OUT_OF_ROAD]
@@ -218,7 +219,7 @@ def test_ma_roundabout_reset():
                     assert len(v.navigation.checkpoints) > 2
 
                 for kkk, iii in i.items():
-                    if iii and iii[TerminationState.SUCCESS]:
+                    if TerminationState.SUCCESS in iii and iii[TerminationState.SUCCESS]:
                         # print("{} success!".format(kkk))
                         success_count += 1
 
@@ -395,11 +396,6 @@ def test_ma_roundabout_reward_done_alignment_1():
                     assert iii[TerminationState.CRASH_VEHICLE]
                 if iii[TerminationState.CRASH_VEHICLE]:
                     assert iii[TerminationState.CRASH]
-                    # #assert r[kkk] == -1.7777
-            for kkk, ddd in d.items():
-                if ddd and kkk != "__all__":
-                    assert i[kkk][TerminationState.OUT_OF_ROAD]
-                    # print('{} done passed!'.format(kkk))
             for kkk, rrr in r.items():
                 if rrr == -1.7777:
                     # assert d[kkk]
