@@ -217,6 +217,15 @@ class Parameter:
 
 
 class VehicleParameterSpace:
+    STATIC_BASE_VEHICLE = dict(
+        wheel_friction=ConstantSpace(0.9),
+        max_engine_force=ConstantSpace(800),
+        max_brake_force=ConstantSpace(150),
+        max_steering=ConstantSpace(40),
+        max_speed=ConstantSpace(80),
+    )
+    STATIC_DEFAULT_VEHICLE = STATIC_BASE_VEHICLE
+
     BASE_VEHICLE = dict(
         wheel_friction=ConstantSpace(0.9),
         max_engine_force=BoxSpace(750, 850),
@@ -225,6 +234,7 @@ class VehicleParameterSpace:
         max_speed=ConstantSpace(80),
     )
     DEFAULT_VEHICLE = BASE_VEHICLE
+
     S_VEHICLE = dict(
         wheel_friction=ConstantSpace(0.9),
         max_engine_force=BoxSpace(350, 550),
@@ -261,6 +271,8 @@ class BlockParameterSpace:
     otherwise, an error may happen in navigation info normalization
     """
     STRAIGHT = {Parameter.length: BoxSpace(min=40.0, max=80.0)}
+    BIDIRECTION = {Parameter.length: BoxSpace(min=40.0, max=80.0)}
+
     CURVE = {
         Parameter.length: BoxSpace(min=40.0, max=80.0),
         Parameter.radius: BoxSpace(min=25.0, max=60.0),
@@ -274,8 +286,8 @@ class BlockParameterSpace:
     }
     ROUNDABOUT = {
         # The radius of the
-        Parameter.radius_exit: BoxSpace(min=5, max=15),  # TODO Should we reduce this?
-        Parameter.radius_inner: BoxSpace(min=15, max=45),  # TODO Should we reduce this?
+        Parameter.radius_exit: BoxSpace(min=5, max=15),
+        Parameter.radius_inner: BoxSpace(min=15, max=45),
         Parameter.angle: ConstantSpace(60)
     }
     T_INTERSECTION = {
@@ -294,7 +306,8 @@ class BlockParameterSpace:
     BOTTLENECK_PARAMETER = {
         Parameter.length: BoxSpace(min=20, max=50),  # the length of straigh part
         Parameter.lane_num: DiscreteSpace(min=1, max=2),  # the lane num increased or descreased now 1-2
-        "bottle_len": ConstantSpace(20)
+        "bottle_len": ConstantSpace(20),
+        "solid_center_line": ConstantSpace(0)  # bool, turn on yellow line or not
     }
     TOLLGATE_PARAMETER = {
         Parameter.length: ConstantSpace(20),  # the length of straigh part
